#!/usr/bin/env ruby
# coding: UTF-8

# TODO:
# - parallel-browser
# 	- show large-table warning also the same in fullscreen 
# 	- CSV-Download and show table should be disabled if there are no candidates
# - align-browser
# 	- show feasible option the same also in fullscreen
# - general
# 	- the JS-scripts could be compressed. a LOT is not DRY!!!



require 'rubygems'			# only required for Ruby < 1.8
require 'bundler/setup'		# add gems from Gemfile to load-path
Bundler.require(:default, :development)	# actually loads all specified gems. Remove :development in production mode
# run 'bundle install' for missing gems. In case of error remove Gemfile.lock
# run 'gem install bundler' if you are missing bundler

# require the model.rb for user and password storing = database mapper
require "./model.rb"




### Our App in an own class. This is a must since Warden failure routine needs backreference to this class
class SinatraCTSAdmin < Sinatra::Base


########## The (fast) local debug (fast MRI ruby startup, non-Tomcat) is done
# when not working with JRuby 
@@local_debug_modus = (not RUBY_PLATFORM == 'java')
if @@local_debug_modus
	Dir.chdir('/home/sasludwi/Hiwi_Leipzig/2015_03_CTS_Jochen/cts_admin_sascha/webapps/cts_admin/WEB-INF')	
end
################################

################################################################################
#####  Helper Methods ##########################################################
################################################################################

helpers do
	
	
	def get_my_tomcat_name
		return Dir.pwd.split("webapps" + File::SEPARATOR)[1].split(File::SEPARATOR)[0]
	end
	
	def get_my_toolsfolder_name
		return get_my_tomcat_name() + "_tools"
	end
	
	def get_webapps_path
		return Dir.pwd.split("webapps" + File::SEPARATOR)[0] + "webapps" 
	end
	
	def get_tools_path
		return get_webapps_path() + File::SEPARATOR + get_my_toolsfolder_name
	end
	
	
	
	def get_settings()
		if File.exists?(@@settings_file)
			# return values from the JSON file
			cont = File.read(@@settings_file)
			return JSON.parse(cont)
		else
			@template_files = Dir.glob("resources/*.war").map{|e| e.gsub("resources/", "")}.sort
			# return some defaults
			return {"urn_separator" => "_", "cts_template" => @template_files.last}
		end
	end
	
	
	# http://ku1ik.com/2010/09/18/open3-and-the-pid-of-the-spawn.html
	def external_open(*args)
		if IO.respond_to?(:popen4)
			# for JRuby only
			IO.popen4(*args)
		else
			# for all other ruby implementations
			open4(*args)
		end
	end
	
	
	# logs events to a logfile
	def log(msg)
		# only write important messages
		begin
			File.open("error.log", "a") do |log_file|	# open log-file in append mode
				s = "[ " + Time.new.to_s() + " ] "		# save the time
				s += msg.to_s()	+ "\n"					# write message + new line for next message
				log_file.write(s)						# write the string
			end											# close the file properly
		rescue => e										# do not use the error function here, because it is using the log function
			debug("[ERROR] in log-method", 1 )			# in case of error; 
			debug( e.message, 1 )						# print message
			debug( e.backtrace.join("\n"), 1)			# and print backtrace
		end
	end
	
	
	def error(msg, e=nil)
		str = "[ERROR] " + msg.to_s()
		debug( str, 1 )										# show message in red
		if e != nil											# if it is an Ruby-Exception
			debug( e.message, 1)							# print message in red
			debug( e.backtrace.join("\n"), 1)				# and print backtrace in red
			log( str + "\n" + e.message + "\n" + e.backtrace.join("\n") )	# and save to logfile
		else 												# if its an manual detected (logical) error
			log( str )										# and save to logfile
		end
	end
	
	
	# colourfull output for debugging purposes on the command line
	def debug(msg, colour=0)
		str = Time.new.strftime("[%T.%L] ") + msg.to_s()
		puts str		 				if colour == 0	# normal
		puts "\e[31m" + str + "\e[0m" 	if colour == 1	# red
		puts "\e[32m" + str + "\e[0m" 	if colour == 2	# green
		puts "\e[33m" + str + "\e[0m" 	if colour == 3	# yellow
		puts "\e[34m" + str + "\e[0m" 	if colour == 4	# blue
		puts "\e[35m" + str + "\e[0m" 	if colour == 5	# magenta
		puts "\e[36m" + str + "\e[0m" 	if colour == 6	# cyan
	end
	
	
	# get all cts-instances in the tomcat webapps folder
	def get_all_cts()
		ret = []
		# iterate through all folder in "webapps"
		Dir.glob(File.join("..", "..", "*")).select {|f| File.directory?(f) }.sort.each do |folder|
			# check for CTS specific files
			if 	File.exist?( File.join(folder, "WEB-INF", "lib", "ctskey.txt") ) or
				File.exist?( File.join(folder, "WEB-INF", "lib", "conf.properties") ) or
				File.exist?( File.join(folder, "WEB-INF", "lib", "CTSStore.jar") ) 
					# add name of the cts and its folder
					ret.push( { name: folder.split(File::SEPARATOR).last , path: folder } )
			end
		end
		return ret
	end
	
	
	# reads the config file between the tokens "conf_start" and "conf_end" and returns an object of all parameter lines and there comments
	def read_config_between(conf_start, conf_end, ctsname)
		# the returned object. Empty at begining. only the last element of the table data will be updated. 
		# init here with empty lines (=parameters) and comment (= comment for all these parameters)
		tbl_data = [ {lines: [], comment: ""} ]
		
		# only take the part between the two strings
		begin
			# opens the config file and read the content as a string
			raw_config_string = File.read( File.join("..", "..", ctsname, "WEB-INF", "lib", "conf.properties") )
			
			# split for the first time. Do start at the beginning of the file if conf_start == "". Thus, don't split at all ;-)
			if conf_start != "" and raw_config_string.include?(conf_start)
				raw_config_string = raw_config_string.split(conf_start)[1]
			end
			# split for the second time
			if conf_end != "" and raw_config_string.include?(conf_end)
				raw_config_string = raw_config_string.split(conf_end)[0]
			end
		rescue => e
			error "in read_config_between: conf_start=#{conf_start}   conf_end=#{conf_end}", e
			raw_config_string = ""
		end
		
		# there are two modes: collecting all comment lines (above all the parameters) 
		# and collecting the parameters for the comment. file starts with comment
		still_collecting_comments = true
		
		begin
			# for each line in the config file
			raw_config_string.split("\n").each do |row|
				# check if it is a parameter. it is a parameter, if it has a '=' in the line and if it does not start with comment character or white-space
				if row.include?("=") and not ["#", "\r", " ", ""].include?(row[0])
					still_collecting_comments = false	# switch mode to parameter collecting mode
					n = row.split("=")[0].strip			# get left = parameter name
					if row.split("=")[1]
						# and right side of the parameter equation. there might be other = in the string. join them back in 
						# e.g. in sourceDir=http://ctstest.informatik.uni-leipzig.de/demo/cts/?request=GetCapabilities
						v = row.split("=")[1..-1].join("=").strip			
					else
						v = ""
					end
					b = (v == "true" or v == "false") 	# check if it is a boolean parameter. will be shown as checkboxes
					tbl_data.last[:lines].push( {name: n, value: v, is_bool: b} )	# update last element in the table data
				else
				# if it is a comment
					if still_collecting_comments
						# if we are still collecting comment lines, add this line to the comment-string
						# but only show comment if it starts with exactly two ## and not more #
						if row.strip[0..1] == "##" and row.strip[2] != "#"
							# show comment line without leading ##
							tbl_data.last[:comment] += row.strip[2..-1] + "<br>"
						end
					else
						# start a new block (comments are stated ABOVE the corresponding parameters. Thus, they will be read first)
						com = ""
						if row.strip[0..1] == "##" and row.strip[2] != "#"
							# only add the first comment if it should be shown (= start with exactly ## )
							com = row.strip[2..-1] + "<br>"
						end
						tbl_data.push( {lines: [], comment: com} )
						still_collecting_comments = true
					end
				end
			end
		
			# remove empty lines in comments: split comment by breaking lines. for each line, remove all unnecessarry whitespaces. only keep non-empty lines
			tbl_data.each{|b| b[:comment] = b[:comment].split("<br>").each{|e| e.gsub("\r", "").strip}.select{|e| e != ""}.join("<br>") }
		rescue => e
			error "while reading configfile.", e
		end
		
		return tbl_data
	end
	
	
	# read the tomcat config file to get an idea about the port and host of the tomcat instance
	# TODO: this might also be done better by just taking the host-url of the cts-admin instance. Parsing is only done quick and dirty
	def tomcat_conf()
		begin
			conf = File.read( File.join("..", "..", "..", "conf", "server.xml") )
			ret = {port: 0, host: ""}
			ret[:port] = conf.split('<Connector port="')[1].split('"')[0]
			ret[:host] = conf.split('<Host name="')[1].split('"')[0]
		rescue Exception => e
			error "while getting tomcat conf", e
		end
		
		return ret
	end
	
	# returns a timestamp of the current time for logging: e.g. "[2015-04-14 14:32:20]"
	def timestamp()
		return Time.now.strftime("[%Y-%m-%d %H:%M:%S]")
	end
	
	# parses a ruby timestamp: e.g. 2015-04-14 14:32:20
	def get_timestamp(line)
		return /\d\d\d\d-\d\d-\d\d \d\d\:\d\d\:\d\d/.match(line)[0]
	end
	# parses a timestamp from the DataPreparation.jar log file: e.g. 2015.04.14 at 14:32:28
	def get_java_timestamp(line)
		md = /\d\d\d\d\.\d\d\.\d\d at \d\d\:\d\d\:\d\d/.match(line)
		ret = md ? md[0].gsub(".", "-").gsub(" at ", " ") : ""
		return ret
	end
	
	# parses the bare message of a ruby log line
	def get_msg(line)
		return line.gsub(/\[.*\]/, "").strip
	end
	# parses the bare message of a DataPreparation.jar log line
	def get_java_msg(line)
		return line.gsub(/\d\d\d\d\.\d\d\.\d\d at \d\d\:\d\d\:\d\d/, "").strip
	end
	
	
	# type could be "info", "success", "warning", "danger" for colours
	def parse_ruby_logfile()
		blocks = []
		ret = []
		insert_start_offset = nil
		insert_file = nil
		
		begin
			# check if an ruby logfile exists
			if File.exist?(@@import_rubylogfile)
				all_lines = File.read(@@import_rubylogfile).split("\n")
				all_lines.each do |line|
					# block limits
					if line.include?("[START]")
						ret = []
					elsif line.include?("[END]")
						blocks.push ret
						ret = []
					end
					
					# single lines
					if line.include?("[INFO]")
						ret.push( {type: "info", time: get_timestamp(line), msg: get_msg(line)} )
					elsif line.include?("[ERROR]")
						ret.push( {type: "danger", time: get_timestamp(line), msg: get_msg(line)} )
					elsif line.include?("[SUCC]")
						ret.push( {type: "success", time: get_timestamp(line), msg: get_msg(line)} )
					elsif line.include?("[WARN]")
						ret.push( {type: "warning", time: get_timestamp(line), msg: get_msg(line)} )
					elsif line.include?("[INSERT]") and line.include?("startoffset=")
						# at this place the content of the DataPreparation.jar logfile should be inserted
						# parse the corresponding java logfile name and the starting offset
						# but wait, if there is a endoffset in the next line. if so, take this offset. if not, read until the end of file
						insert_start_offset = line.split("startoffset=")[1].strip.to_i
						insert_file = line.split("file=")[1].split("startoffset=")[0].strip
					end
					
					# insert the content of the java log file. this happens if we read an endoffset= 
					# or if this line (could be an startoffset= line) is the last line. this case happens if the import is still running
					if (line.include?("[INSERT]") and line.include?("endoffset=")) or (all_lines.last == line and not insert_start_offset.nil?)
						if line.include?("endoffset=")
							# parse the end offset from the line
							insert_end_offset = line.split("endoffset=")[1].strip.to_i
						else
							# if there is no end offset, read until end of file
							if File.exist? insert_file
								insert_end_offset = File.size?(insert_file)
							else
								insert_end_offset = 0
							end
						end
						
						# check if this file exist. Could not exist, if the cts has been renamed afterwards
						if File.exist?(insert_file)
							# read(name, [length [, offset]] [, opt] )
							file_cont = File.read(insert_file, insert_end_offset.to_i-insert_start_offset.to_i, insert_start_offset.to_i)
							if file_cont
								file_cont.split("\n").each do |ins_line|
									# skip the *****START***** line and do not add non-empty lines
									if get_java_msg(ins_line).strip != "*****START*****" and get_java_msg(ins_line).strip != ""
										# if there is a new timestamp, add as a new line. if not, append to last line. e.g. for java error stacks
										if get_java_timestamp(ins_line) != ""
											if @@error_phrases.any? { |phrase| get_java_msg(ins_line).include? phrase }
												# different colour if there was an error
												ret.push( {type: "danger", time: get_java_timestamp(ins_line), msg: get_java_msg(ins_line) } )
											elsif @@warn_phrases.any? { |phrase| get_java_msg(ins_line).include? phrase }
												# different colour if there was an error
												ret.push( {type: "warning", time: get_java_timestamp(ins_line), msg: get_java_msg(ins_line) } )
											else
												# default case is normal colour
												ret.push( {type: "info", time: get_java_timestamp(ins_line), msg: get_java_msg(ins_line) } )
											end
										else
											ret.last[:msg] += "<br>" + get_java_msg(ins_line)
										end
									end
								end 
							end
						else
							ret.push( {type: "warning", time: get_timestamp(timestamp()), msg: "The cts logfile does not exist anymore. This is probably due to renaming of the CTS." } )
						end
						# java log file has been read
						insert_start_offset = nil
					end
				end
				
				# this happens if the (last) import has not finished yet. Then, there is no [END] at the end of the block. add the dataPreparation logfile line anyway
				if not ret.empty?
					blocks.push ret
				end
			end
		rescue => e
			error "while parsing ruby log file", e
		end
		
		return blocks
	end
	
end



################################################################################
#####  Sinatra Setup - mainly authentication stuff  ##########################
################################################################################

# basic folder and file variables for import stuff. Expand all paths since in import base dir changes
@@import_rubybasedir 	= File.expand_path(Dir.pwd)
@@import_rubylogfile 	= File.expand_path(File.join(Dir.pwd, "import.log"))
@@import_rubylock 	 	= File.expand_path(File.join(Dir.pwd, "import.lock"))
@@helpmenu_entriesfile 	= File.expand_path(File.join(Dir.pwd, "helpmenu.json"))
@@settings_file 		= File.expand_path(File.join(Dir.pwd, "settings.json"))
# always nil but when importing
@@import_ctsname 	= nil
@@import_ctsdir 	= nil
@@import_ctslogfile = nil

# these phrases in a log line indicating errors
@@error_phrases 	= ["Access denied", "Can't create database", "Exception"]
@@warn_phrases 		= ["Duplicate entry", "canceled"]

# enable session for user-specific variables. Needed for Warden authentication
enable :sessions
# TODO remove!!!
# set :session_secret, 'supersecret454sdfasd2fd544222'
# flash messages for feedback
register Sinatra::Flash



use Warden::Manager do |config|
	# Tell Warden how to save our User info into a session.
	# Sessions can only take strings, not Ruby code, we'll store the User's `id`
	config.serialize_into_session{ |user| user.id }
	# Now tell Warden how to take what we've stored in the session
	# and get a User from that information.
	config.serialize_from_session{ |id| User.get(id) }

	config.scope_defaults :default,
	# "strategies" is an array of named methods with which to
	# attempt authentication. We have to define this later.
	strategies: [:password],
	# The action is a route to send the user to when
	# warden.authenticate! returns a false answer. We'll show
	# this route below.
	action: 'auth/unauthenticated'
	# When a user tries to log in and cannot, this specifies the
	# app to send the user to.
	config.failure_app = self
end


Warden::Manager.before_failure do |env, opts|
	env['REQUEST_METHOD'] = 'POST'
end


# write the code for the :password strategy we called above
# In the following block, they keys of params which I am using are based on 
# the login form I made.
Warden::Strategies.add(:password) do
	def valid?
		(not params.empty?) && params['login_username'] && params['login_password']
	end

	def authenticate!
		ap params
		ap User.all
		# User class comes from the datamapper class file!
		user = User.first( username: params['login_username'] )
		
		ap user
		
		if user.nil?
			throw(:warden, message: "The username you entered does not exist.")
		elsif user.authenticate( params['login_password'] )
			success!(user)
		else
			throw(:warden, message: "The username and password combination does not fit")
		end
	end
end




################################################################################
#####  Routing  = Main Program Logic  ##########################################
################################################################################

# on error
error do
	msg = env['sinatra.error'].message
	error "general error: #{msg}"
end

# if something is not found, go to index page
not_found do
	redirect to '/'
end


# before each route is executed, get the list of all cts and the current user
before do
	begin
		if not ["/auth/login", '/auth/logout', '/auth/unauthenticated'].include?( request.env["PATH_INFO"] )
			# needs authentication
			env['warden'].authenticate!
		end
		
		@helpmenu_entries = []
		if File.exists?(@@helpmenu_entriesfile)
			@helpmenu_entries = JSON.parse( File.read(@@helpmenu_entriesfile) )
		end
		@base_url		= url('/')[0...-1]
		@all_cts 		= get_all_cts()
		@current_user 	= env['warden'].user.username if env['warden'].user
	rescue Exception => e
		error(e)
	end
end


# this page shows the main page without any collection or action specified
get '/' do
	# show stuff 
	@sub_erb = :index
	erb :main
end


# create a new cts instance
get '/create_new_cts' do
	# show stuff
	@sub_erb = :cts_create_new
	erb :main
end



# upload a new cts file
get '/update_template' do
	# data
	@up_ret		= nil
	# show stuff
	@sub_erb	= :cts_upload_template
	erb :main
end



# shows an administration area
get '/admin_area' do 
	
	# settings
	@settings = get_settings()
	
	# specific data
	# User.all is called in the erb-file
	@json = ""
	if File.exists?(@@helpmenu_entriesfile)
		@json = CGI.escapeHTML(File.read(@@helpmenu_entriesfile))
	end
	
	###### size of all databases
	# 1) read the config of all known CTS-instances 
	mapping = {}
	mysqls 	= []
	@all_cts.each do |cts|
		mysql = {}
		# read the data base config part of the config file
		tbl_data = read_config_between("", "#IMPORT#", cts[:name])
		tbl_data.each do |blck|
			blck[:lines].each do |line|
				if line[:name] == "db_host"
					mysql[:host] = line[:value]
				elsif line[:name] == "db_port"
					mysql[:port] = line[:value]
				elsif line[:name] == "db_pw"
					mysql[:password] = line[:value]
				elsif line[:name] == "db_name"
					mapping[line[:value]] = cts[:name]
				elsif line[:name] == "db_user"
					mysql[:username] = line[:value]
				end
			end
		end
		mysqls.push mysql
	end
	
	# 2) get the table schemes for all found unique MySQL connections
	size_query = """ SELECT table_schema 'db_name',
	sum( data_length + index_length ) / 1024 /
	1024 'size_MB'
	FROM information_schema.TABLES
	GROUP BY table_schema ; """
	
	@size_arr = []
	mysqls.uniq!
	mysqls.each do |con|
		begin
			batch_query = "mysql --batch --column-names=false --user=\"#{con[:username]}\" --password=\"#{con[:password]}\" --host=#{con[:host]} --port=#{con[:port]} --execute=\"#{size_query.gsub("\n", " ")}\""
			mysql_result = %x(#{batch_query})
			mysql_result.split("\n").each do |row|
				spl = row.split("\t")
				@size_arr.push( {"db_name" => spl[0], "size_MB" => spl[1], "mysql" => con} )
			end
		rescue Exception => e
			error e
		end
	end
	
	# 3) perform a unique operator above the databases
	@size_arr.uniq!
	@size_arr.delete_if{|e| ["mysql", "information_schema", "performance_schema"].include? e["db_name"]}
	
	# 4) match with CTS-instances
	@size_arr.map! do |e|
		e["cts"] = mapping[e["db_name"]] || ""
		e
	end
	#####
	
	# CTS-Templates list
	@template_files = Dir.glob("resources/*.war").map{|e| e.gsub("resources/", "")}.sort.reverse
	
	
	# show stuff
	@sub_erb = :admin_area
	erb :main
end



post "/ajax_helpmenu_save" do
	begin
		File.open(@@helpmenu_entriesfile, "w") do |f|
			f.write params[:text]
		end
		return "OK"
	rescue Exception => e
		return e.message
	end
end



post "/ajax_settings_save" do
	ap params
	begin
		settings = get_settings()
		settings["urn_separator"] = params[:urn_separator]
		File.open("settings.json", "w") do |file|
			file.write(JSON.pretty_generate( settings ))
		end
		return "OK"
	rescue Exception => e
		return e.message
	end
end



post "/ajax_cts_template_save" do
	ap params
	begin
		settings = get_settings()
		settings["cts_template"] = params["cts_template"]
		File.open("settings.json", "w") do |file|
			file.write(JSON.pretty_generate( settings ))
		end
		return "OK"
	rescue Exception => e
		return e.message
	end
end



post "/ajax_cts_template_delete" do
	ap params
	ret = {succ: false, msg: "", new_active: ""}
	begin
		template_files = Dir.glob("resources/*.war").map{|e| e.gsub("resources/", "")}.sort.reverse
		# 1) check if last one would be deleted
		if(template_files.size > 1)
			# 2) check if active one got deleted --> set another one
			settings = get_settings()
			if settings["cts_template"] == params["cts_template"]
				new_template_fn = template_files.delete_if{|e| e == params["cts_template"]}.last
				ret[:new_active] = new_template_fn;
				settings["cts_template"] = new_template_fn
				File.open("settings.json", "w") do |file|
					file.write(JSON.pretty_generate( settings ))
				end
			end
			# 3) delete the given template
			File.delete( File.join("resources", params["cts_template"]) )
			ret = {succ: true, msg: params["cts_template"] + " has been deleted successfully!", new_active: ""}
		else
			ret = {succ: false, msg: "You cannot delete the last template!", new_active: ""}
		end
	rescue Exception => e
		ret = {succ: false, msg: e.message, new_active: ""} 
	end
	
	return ret.to_json
end



post "/ajax_user_save" do
	ret = {succ: false, msg: ""}
	
	if User.all(role: "Admin").length == 1 and User.first(username: params[:uname]).role == "Admin" and not params[:role] == "Admin"
		ret[:msg] = "There must be at least one user with the role 'Admin'"
	else
		if params[:new_pw] != "" or params[:new_pw2] != ""
			if not params[:new_pw] == params[:new_pw2]
				ret[:msg] = "New passwords do not match!"
			else
				begin
					User.first(username: params[:uname]).update(role: params[:role], password: params[:new_pw])
					ret = {succ: true, msg: "User '#{params[:uname]}' updated successfully"}
				rescue Exception => e
					ret[:msg] = e.message
				end
			end
		else
			begin
				User.first(username: params[:uname]).update(role: params[:role])
				ret = {succ: true, msg: "User '#{params[:uname]}' updated successfully"}
			rescue Exception => e
				ret[:msg] = e.message
			end
		end
	end
	
	return ret.to_json
end



post "/ajax_user_create" do
	new_id = User.all.map { |e| e.id }.max + 1
	ret = {succ: false, msg: ""}
	
	if /[^a-zA-Z0-9_]/ =~ params[:uname]
		ret[:msg] = "Only a-z A-Z 0-9 and _ are allowed characters!"
	else
		if params[:uname].length < 3
			ret[:msg] = "Username must at least be 3 characters long"
		else
			if not User.first(username: params[:uname]).nil?
				ret[:msg] = "Username already been taken!"
			else
				if params[:new_pw].length < 3 or params[:new_pw2].length < 3
					ret[:msg] = "Password must at least be 3 characters long"
				else 
					if not params["new_pw"] == params["new_pw2"]
						ret[:msg] = "Passwords do not match!"
					else
						begin
							# set id manually due to bug in yaml dm store !!!
							User.create(id: new_id, username: params[:uname], password: params[:new_pw], role: params[:role])
							ret = {succ: true, msg: "User '#{params[:uname]}'' created successfully!"}
						rescue Exception => e
							ret[:msg] = e.message
						end
					end
				end
			end
		end
	end
	
	return ret.to_json
end



post "/ajax_user_delete" do
	ret = {succ: false, msg: ""}
	
	if User.all.length == 1
		ret[:msg] = "You cannot delete all users! Otherwise you cannot access this portal anymore!"
	else
		if User.all(role: "Admin").length == 1 and User.first(username: params[:username]).role == "Admin"
			ret[:msg] = "You cannot delete the last user with role 'Admin'"
		else
			begin
				User.first(username: params[:username]).destroy
				ret = {succ: true, msg: "User '#{params[:username]}' deleted successfully!"}
			rescue Exception => e
				ret[:msg] = e.message
			end
		end
	end
	
	return ret.to_json
end





# Handle POST-request (Receive and save the uploaded file)
# the uploaded file will be stored as a temp-file
# "cts_file" => {
#     :filename => "wallpaper.jpg",
#         :type => "image/jpeg",
#         :name => "cts_file",
#     :tempfile => #<Tempfile:/tmp/RackMultipart20150406-7362-25i5l6.jpg>,
#         :head => "Content-Disposition: form-data; name=\"cts_file\"; filename=\"wallpaper.jpg\"\r\nContent-Type: image/jpeg\r\n"
# }
post "/update_template" do 
	# get the file-hash for the uploaded file (=input id) 
	fh = params["cts_file"]
	# default return if nothing bad happens
	@up_ret = {succ: true, msg: "CTS template updated successfully"}
	
	begin
		# check the filename
		if fh[:filename] == "cts.war"
			begin
				# rename the old cts.war with timestamp
				# File.rename( File.join("resources", "cts.war"), File.join("resources", "cts_" + Time.now.strftime("%Y%m%d_%H%M%S")) + ".war" )
				# copy the new uploaded file to /resources/cts.war
				fn = "cts_" + Time.now.strftime("%Y%m%d_%H%M%S") + ".war"
				FileUtils.cp( fh[:tempfile].path, File.join("resources", fn) )
				# set the new file in the settings
				settings = get_settings()
				settings["cts_template"] = fn
				File.open("settings.json", "w") do |file|
					file.write(JSON.pretty_generate( settings ))
				end
			rescue => e
				# if error, display this error
				@up_ret = {succ: false, msg: "[ERROR] " + e.message}
			end
		else
			# if the filename does not match, give feedback
			@up_ret = {succ: false, msg: "Filename must be cts.war!"}
		end
	rescue => e
		error "while updating the CTS template", e
	end
	
	# show stuff
	@sub_erb		= :cts_upload_template
	erb :main
end








################################################################################
#####  Main Routes  ############################################################
################################################################################








# fixed url to select collections and actions
# matches /collection/hello/action/world
# also possible instead of block: params[:splat] # => [hello, world]
get '/forcts/*/action/*' do |ctsname, action|
	# get parameter
	@ctsname 	= ctsname
	@action 	= action
	
	begin
		# important state variables
		@import_runnig  = File.exist?( @@import_rubylock )
		@import_ctsname = @import_runnig ? File.read(@@import_rubylock) : ""
		
		# first check if the specified cts in the url exists. Then, show action
		if get_all_cts().map{ |e| e[:name] }.include?(ctsname)
			if @action == "db_config"
				# read the data base config part of the config file
				@tbl_data 	= read_config_between("", "#IMPORT#", @ctsname)
				@sub_erb 	= :cts_db_config	# show the config site
			elsif @action == "import"
				# read the import settings and the log-files of recent import activities. also check if an import is currently running
				@tbl_data 	= read_config_between("#IMPORT#", "#PARAMETERS#", @ctsname)
				@log_lines 	= parse_ruby_logfile()
				@sub_erb 	= :cts_import		# show the import site
			elsif @action == "servlet"
				# read the servlet parameters
				@tbl_data 	= read_config_between("#PARAMETERS#", "", @ctsname)
				@sub_erb 	= :cts_servlet		# show the servlet site
			elsif @action == "browse_capa"
				@capa 		= "something went wrong. Probably something with Tomcat."
				begin
					if @@local_debug_modus
						tc 		= tomcat_conf()
						cts_url = "http://" + tc[:host] + ":" + tc[:port] + "/" + @ctsname + "/cts/?request=GetCapabilities"
					else
						cts_url = url( "../" + @ctsname + "/cts/?request=GetCapabilities" )
					end
					@capa 	= CGI.escapeHTML(open( cts_url ).read)
				rescue
					@capa 	= "CTS is not responding in cts get capabilities. #{cts_url}"
				end
				@sub_erb 	= :cts_browse_capa		# browse GetCapabilities
			elsif @action == "browse_editions"
				@editions 		= "something went wrong. Probably something with Tomcat."
				begin
					if @@local_debug_modus
						tc 		= tomcat_conf()
						cts_url = "http://" + tc[:host] + ":" + tc[:port] + "/" + @ctsname + "/plain/editions"
					else
						cts_url = url( "../" + @ctsname + "/plain/editions" )
					end
					@editions 	= CGI.escapeHTML(open( cts_url ).read)
					@editions.gsub!("\t", "&nbsp"*4) if @editions
				rescue
					@editions 	= "CTS is not responding in cts get editions. #{cts_url}"
				end
				@sub_erb 	= :cts_browse_editions		# browse editions
			elsif @action == "browse_browser"
				# http://ctstest.informatik.uni-leipzig.de/browser/?ctsURL=../../dta/cts/
				@browserdir_exists = File.exist?(File.join(get_tools_path(), "browser"))
				begin
					if @@local_debug_modus
						tc 			 = tomcat_conf()
						@browser_url = "http://" + tc[:host] + ":" + tc[:port] + "/" + get_my_toolsfolder_name + "/browser/?ctsURL=../../" + @ctsname + "/cts/" 
					else
						@browser_url = "/" + get_my_toolsfolder_name + "/browser/?ctsURL=../../" + @ctsname + "/cts/"
					end
				rescue => e
					error "Tomcat not responding in cts_browse_browser", e
				end
				@sub_erb 	= :cts_browse_browser	# browse Browser
			elsif @action == "browse_align"
				@browserdir_exists = File.exist?(File.join(get_tools_path(), "alignbrowser")) and
									 File.exist?(File.join(get_tools_path(), "Alignment"))
				begin
					if @@local_debug_modus
						tc 			 	   = tomcat_conf()
						@align_browser_url = "http://" + tc[:host] + ":" + tc[:port] + "/" + get_my_toolsfolder_name + "/alignbrowser/?ctsURL=../../" + @ctsname
					else
						@align_browser_url = "/" + get_my_toolsfolder_name + "/alignbrowser/?ctsURL=../../" + @ctsname
					end
				rescue => e
					error "Tomcat not responding in cts_browse_align", e
				end
				@sub_erb 	= :cts_browse_align			# browse Alignment
			elsif @action == "browse_parallel"
				@browserdir_exists = File.exist?(File.join(get_tools_path(), "parallelbrowser"))
				@urn_separator = get_settings()["urn_separator"]
				begin
					if @@local_debug_modus
						tc 			 = tomcat_conf()
						@parallel_browser_url = "http://" + tc[:host] + ":" + tc[:port] + "/" + get_my_toolsfolder_name + "/parallelbrowser/?ctsURL=../../" + @ctsname + "&sep=" + @urn_separator 
					else
						@parallel_browser_url = "/" + get_my_toolsfolder_name + "/parallelbrowser/?ctsURL=../../" + @ctsname + "&sep=" + @urn_separator 
					end
				rescue => e
					error "Tomcat not responding in cts_browse_parallel", e
				end
				@sub_erb 	= :cts_browse_parallel		# browse parallel Alignment
			else
				@sub_erb 	= :index
			end
			
		else
			@sub_erb = :index
		end
	rescue => e
		error "in /forcts/*/action/* route", e
	end
	
	# show
	erb :main
end



# the config parameters should be saved now. This is generic for all cts_template_parameter_tablebody.erb tables
post '/ajax_save_parameters' do 
	begin
		if params[:name] and params[:values]
			# default return if everything is fine
			ret = {success: true, msg: "parameters saved successfully!"}
			
			# read the old-content = whole file
			s = File.read( File.join("..", "..", params[:name].strip, "WEB-INF", "lib", "conf.properties") )
			
			# only replace the lines with parameters and keep the rest untouched
			params[:values].each do |r|
				# construct the new line
				nl = r[1][:name].strip + "=" + r[1][:value].strip
				reg = /#{r[1][:name].strip}\s*=.*$/
				s.gsub!(reg, nl)
			end
			
			# try to open the config file and overwrite it
			begin
				File.open( File.join("..", "..", params[:name].strip, "WEB-INF", "lib", "conf.properties"), 'w' ) do |f|
					f.write(s)
				end
			rescue => e
				ret = {success: false, msg: "[ERROR] " + e.message}
			end
		else
			ret = {success: false, msg: "[ERROR] empty parameters or no cts-name given!"}
		end
	rescue Exception => e
		error "in /ajax_save_parameters", e
	end
	
	# return ret object as a json string
	return ret.to_json 
end



post '/ajax_drop_table' do
	ret = {succ: false, msg: ""}
	mysql = {}
	
	begin
		# read the data base config part of the config file
		tbl_data = read_config_between("", "#IMPORT#", params["name"])
		tbl_data.each do |blck|
			blck[:lines].each do |line|
				if line[:name] == "db_host"
					mysql[:host] = line[:value]
				elsif line[:name] == "db_port"
					mysql[:port] = line[:value]
				elsif line[:name] == "db_pw"
					mysql[:password] = line[:value]
				elsif line[:name] == "db_name"
					mysql[:database] = line[:value]
				elsif line[:name] == "db_user"
					mysql[:username] = line[:value]
				end
			end
		end
		
		# connect to mysql and drop table
		# db = Mysql2::Client.new(:host => mysql[:host], :username => mysql[:user], :password => mysql[:pw], :port => mysql[:port])
	 	# ap db.methods - Object.methods
	 	drop_query = "DROP DATABASE IF EXISTS #{mysql[:database]}"
	 	batch_query = "mysql --batch --column-names=false --user=\"#{mysql[:username]}\" --password=\"#{mysql[:password]}\" --host=#{mysql[:host]} --port=#{mysql[:port]} --execute=\"#{drop_query.gsub("\n", " ")}\""
		mysql_result = %x(#{batch_query + " 2>&1"})
		if mysql_result.include?("ERROR")
			ret = {succ: false, msg: mysql_result}
		else
	 		ret = {succ: true, msg: "Database has successfully been droped!"}
	 	end
 	rescue Exception => e
		ret = {succ: false, msg: "[ERROR] #{e.message}"}
	end
 	
	return ret.to_json
end





post "/ajax_database_delete" do 
	ret = {succ: false, msg: ""}
	begin
	 	drop_query   = "DROP DATABASE IF EXISTS #{params["database"]}"
	 	batch_query  = "mysql --batch --column-names=false --user=\"#{params[:username]}\" --password=\"#{params[:password]}\" --host=#{params[:host]} --port=#{params[:port]} --execute=\"#{drop_query.gsub("\n", " ")}\""
		mysql_result = %x(#{batch_query + " 2>&1"})
		if mysql_result.include?("ERROR")
			ret = {succ: false, msg: mysql_result}
		else
	 		ret = {succ: true, msg: "Database '#{params["database"]}' has successfully been deleted!"}
	 	end
 	rescue Exception => e
		ret = {succ: false, msg: "[ERROR] #{e.message}"}
	end
	
	return ret.to_json
end



post "/ajax_restart_tomcat" do
	ret = {succ: false, msg: ""}
	begin
		pid, stdin, stdout, stderr = external_open("echo #{params["password"]} | sudo -S service tomcat7 restart")
		
		stdo = stdout.read.gsub("\n", "<br>")
		stde = stderr.read.gsub("\n", "<br>")
		debug stdo, 2
		debug stde, 1
		
		if(stde == "")
			ret[:succ] = true
			ret[:msg]  = stdo
		else
			ret[:succ] = false
			ret[:msg]  = stdo + "<br>" if stdo != ""
			ret[:msg] += stde
		end
	rescue Exception => e
		ret = {succ: false, msg: "[ERROR] #{e.message}"}
	end
	
	return ret.to_json
end


post "/ajax_restart_tomcat_local" do
	ret = {succ: false, msg: ""}
	begin
		bindir = Dir.pwd.split("webapps")[0] + "bin/"
		cmd =  bindir + "shutdown.sh && sleep 5 && " + bindir + "startup.sh"
		pid, stdin, stdout, stderr = external_open( cmd )
		
		stdo = stdout.read.gsub("\n", "<br>")
		stde = stderr.read.gsub("\n", "<br>")
		debug stdo, 2
		debug stde, 1
		
		if(stde == "")
			ret[:succ] = true
			ret[:msg]  = stdo
		else
			ret[:succ] = false
			ret[:msg]  = stdo + "<br>" if stdo != ""
			ret[:msg] += stde
		end
	rescue Exception => e
		ret = {succ: false, msg: "[ERROR] #{e.message}"}
	end
	
	return ret.to_json
end


# delete an CTS instance. Remove .war file and give tomcat 30s to remove the folder
post '/ajax_delete_cts' do
	# get the name of the cts
	n = params[:name].strip
	# default return
	ret = {success: false, msg: "[ERROR] Timeout. Tomcat has not removed the CTS folder in time."}
	
	begin
		# delete the WAR file
		fn = File.join("..", "..", n + ".war" )
		debug "Now deleting .war file #{fn}", 2
		File.delete(fn)
				
		# wait a maximum of 30s until Tomcat has removed folder
		sec = 30
		sec.times do |i|
			sleep 1
			debug "Waiting for Tomcat to remove CTS folder. waited seconds = #{i+1}", 2
			# break this waiting loop if folder has been removed
			if not File.exist?(File.join("..", "..", n))
				ret = {success: true, msg: "CTS '#{n}' has been deleted successfully!" + ' This page will be reloaded automatically in 5 seconds to apply the changes. <a href="' + url("/") + '" class="alert-link">Reload now!</a>', to: url("/") }
				break 
			end
		end
	rescue => e
		ret = {success: false, msg: "[ERROR] " + e.message}
		error "in /ajax_delete_cts", e
	end
	
	return ret.to_json
end



# rename an CTS instance
post '/ajax_rename_cts' do
	# default return value
	ret = {success: false, msg: "[ERROR] Timeout. Tomcat has not added the renamed CTS folder in time."}
	
	# get old_name and new_name
	on = params[:old_name]
	nn = params[:new_name]
	
	begin
		# make the new name a sane string: only a-z A-Z 0-9 and _ is allowed
		nn.gsub!(/[^a-zA-Z0-9_]/, "")
		# check, that the name is not already in use
		if get_all_cts().map{ |e| e[:name] }.include?(nn)
			ret = {success: false, msg: "[ERROR] Name conflict. There is already another CTS instance with this name!"}
		else
			if get_my_toolsfolder_name() == nn
				ret = {success: false, msg: "[ERROR] Name conflict. Name must not be: " + get_my_toolsfolder_name() }
			else
				if Dir.glob(File.join("..", "..", "*")).select{|f| File.directory?(f) }.map{ |e| e.split(File::Separator).last }.include?(nn)
					ret = {success: false, msg: "[ERROR] Name conflict. There is already another subfolder with this name in the Tomcat webapps folder." }
				else
					# fist backup the conf.properties, because they will be overwritten
					conf_backup = File.read( File.join("..", "..", on, "WEB-INF", "lib", "conf.properties") )
					# rename the .war file
					File.rename( File.join("..", "..", on + ".war"), File.join("..", "..", nn + ".war") )
					# and wait at most 30s for Tomcat to deploy
					30.times do |i|
						sleep 1
						debug "Waiting for Tomcat to rename CTS folder. waited seconds = #{i+1}", 2
						if File.exist?(File.join("..", "..", nn))
							begin
								# restore the backupped conf.properties
								File.open(File.join("..", "..", nn, "WEB-INF", "lib", "conf.properties"), 'w') do |f|
									f.write conf_backup
								end
								ret = {success: true, msg: 'CTS has been renamed successfully! This page will be reloaded automatically in 5 seconds to apply the changes. <a href="' + url("/forcts/#{nn}/action/servlet") + '" class="alert-link">Reload now!</a>', to: url("/forcts/#{nn}/action/servlet") }
							rescue => e
								ret = {success: false, msg: "[ERROR] " + e.message}
							end
							# break the loop
							break 
						end
					end
				end
			end
		end
	rescue => e
		error "in /ajax_rename_cts", e
	end
	
	# return the values
	return ret.to_json
end 



# rename an CTS instance
post '/ajax_update_cts' do
	# default return value
	ret = {success: false, msg: "[ERROR] Timeout. Tomcat has not updated the CTS folder in time."}
	
	# get cts_name
	cts_name = params[:cts_name]
	
	begin
		# fist backup the conf.properties and the Log.txt, because they will be overwritten
		conf_backup = File.read( File.join("..", "..", cts_name, "WEB-INF", "lib", "conf.properties") )
		if File.exists? File.join("..", "..", cts_name, "WEB-INF", "lib", "Log.txt")
			log_backup  = File.read( File.join("..", "..", cts_name, "WEB-INF", "lib", "Log.txt") )
		else
			log_backup = nil
		end
		
		### delete the WAR file
		fn = File.join("..", "..", cts_name + ".war" )
		debug "Now deleting .war file #{fn}", 2
		File.delete(fn)
				
		# wait a maximum of 30s until Tomcat has removed folder
		sec = 30
		sec.times do |i|
			sleep 1
			debug "Waiting for Tomcat to remove CTS folder. waited seconds = #{i+1}", 2
			# break this waiting loop if folder has been removed
			if not File.exist?(File.join("..", "..", cts_name))
				break 
			end
		end
		
		if not File.exist?(File.join("..", "..", cts_name))
			### deploy new cts template file
			FileUtils.cp( File.expand_path(File.join("resources", get_settings()["cts_template"])), File.expand_path(File.join("..", "..", cts_name + ".war")) )
			# and wait at most 30s for Tomcat to deploy
			30.times do |i|
				sleep 1
				debug "Waiting for Tomcat to deploy CTS folder. waited seconds = #{i+1}", 2
				if File.exist?(File.join("..", "..", cts_name))
					begin
						# restore the backupped conf.properties
						File.open(File.join("..", "..", cts_name, "WEB-INF", "lib", "conf.properties"), 'w') do |f|
							f.write conf_backup
						end
						# restore the backupped import log
						if log_backup
							File.open(File.join("..", "..", cts_name, "WEB-INF", "lib", "Log.txt"), 'w') do |f|
								f.write log_backup
							end
						end
						ret = {success: true, msg: 'CTS has been updated successfully!' }
					rescue => e
						ret = {success: false, msg: "[ERROR] " + e.message}
					end
					# break the loop
					break 
				end
			end
		else
			ret = {success: true, msg: 'CTS has not been updated due to Tomcat timeout' }
		end
		
	rescue => e
		error "in /ajax_update_cts", e
		ret = {success: false, msg: e.message}
	end
	
	# return the values
	return ret.to_json
end 



# create a new CTS instance by copying the cts.war template to the webapps folder
post '/ajax_create_new_cts' do
	nn = params[:new_name]
	ret = {success: false, msg: "[ERROR] Timeout. Tomcat has not created CTS folder in time."}
	
	# make the new name a sane string: only a-z A-Z 0-9 and _ is allowed
	nn.gsub!(/[^a-zA-Z0-9_]/, "")
	# check, that the name is not already in use
	if get_all_cts().map{ |e| e[:name] }.include?(nn)
		ret = {success: false, msg: "[ERROR] Name conflict. There is already another CTS instance with this name!"}
	else
		if get_my_toolsfolder_name() == nn
			ret = {success: false, msg: "[ERROR] Name conflict. Name must not be: " + get_my_toolsfolder_name() }
		else
			if Dir.glob(File.join("..", "..", "*")).select{|f| File.directory?(f) }.map{ |e| e.split(File::Separator).last }.include?(nn)
				ret = {success: false, msg: "[ERROR] Name conflict. There is already another subfolder with this name in the Tomcat webapps folder." }
			else
				# copy template file
				begin
					FileUtils.cp( File.expand_path(File.join("resources", get_settings()["cts_template"])), File.expand_path(File.join("..", "..", nn + ".war")) )
					# and wait at most 30s for Tomcat to deploy
					30.times do |i|
						sleep 1
						debug "Waiting for Tomcat to create CTS folder. waited seconds = #{i+1}", 2
						 if File.exist?(File.join("..", "..", nn))
							ret = {success: true, msg: 'CTS has been created successfully! Go to <a href="' + url("/forcts/#{nn}/action/db_config") + '" class="alert-link">db-config page</a>'}
							break
						end
					end
				rescue => e
					ret = {success: false, msg: "[ERROR] " + e.message}
					error "in /ajax_create_new_cts", e
				end
			end
		end
	end
	
	return ret.to_json
end 



# checks the value of the parameter resetDB. Needed for two-stage button
post '/ajax_get_para_resetdb' do
	ret = {reset: false, dbname: ""}
	begin
		raw_config_string = File.read( File.join("..", "..", params[:name].strip, "WEB-INF", "lib", "conf.properties") )
		ret[:reset]  = raw_config_string.split("resetDB")[1].split("\n")[0].gsub("=", "").strip
		ret[:dbname] = raw_config_string.split("db_name")[1].split("\n")[0].gsub("=", "").strip
	rescue => e
		error "in /ajax_get_para_resetdb", e
	end

	return ret.to_json
end



# imports data to the database as specified in the configfile
post "/ajax_import_data" do 
	# default return
	ret = {success: true, msg: "Everything is fine!"}
	begin
		# write a start marker
		File.open(@@import_rubylogfile, "a") do |file| 
			file.write " [START] #{timestamp}\n"
		end
	rescue => e
		error "could not write to ruby logfile '#{@@import_rubylogfile}'", e
	end
	
	
	begin
		if not File.exist?( @@import_rubylock )
			# set some paths. expand the paths since relative path could change since base-dir changes to run java-file
			@@import_ctsname 	 = params[:name].strip
			@@import_ctsdir 	 = File.expand_path( File.join("..", "..", @@import_ctsname, "WEB-INF", "lib"))
			@@import_ctslogfile  = File.expand_path( File.join("..", "..", @@import_ctsname, "WEB-INF", "lib", "Log.txt"))
			File.open(@@import_rubylogfile, "a") do |file| 
				# write some basic information
				file.write "  [INFO] #{timestamp} starting to import data for '#{@@import_ctsname}'\n"
				file.write "[HIDDEN] #{timestamp} @@import_rubybasedir = #{@@import_rubybasedir}\n"
				file.write "[HIDDEN] #{timestamp} @@import_rubylogfile = #{@@import_rubylogfile}\n"
				file.write "[HIDDEN] #{timestamp} @@import_rubylock    = #{@@import_rubylock}\n"
				file.write "[HIDDEN] #{timestamp} @@import_ctsname     = #{@@import_ctsname}\n"
				file.write "[HIDDEN] #{timestamp} @@import_ctsdir      = #{@@import_ctsdir}\n"
				file.write "[HIDDEN] #{timestamp} @@import_ctslogfile  = #{@@import_ctslogfile}\n"
			end
			
			### trigger import process
			# go to the lib-dir to start the java file from there!
			Dir.chdir(@@import_ctsdir)
			
			# start a new thread to execute the java command
			# no need to join this thread at the end since sinatra will not stop running and thus this thread is kept alive
			@@import_thread = Thread.new do
				begin
					debug "starting import!", 2
					# set lock
					File.open(@@import_rubylock, "w") { |file| file.write @@import_ctsname }
					# get the current file of the java log file. Now we now, from which byte on the log-file corresponds to this action!
					start_off = File.size?(@@import_ctslogfile).to_i
					File.open(@@import_rubylogfile, "a") do |file|
						file.write "[INSERT] #{timestamp} file=#{@@import_ctslogfile} startoffset=#{start_off.to_s}\n" 
					end
					
					
					####################################################################################
					### execute java (old: and redirect STDERR-Output via "2>&1") ######################
					####################################################################################
					@@stderr 	= nil
					@@stdin 	= nil
					@@stdout 	= nil
					@@java_pid 	= nil
					@@java_thread_got_killed = false
					@@java_thread = Thread.new do
						# no need for "2>&1"-redirection since we get all streams. redirection would force for false PID's anyway. use exec when using redirections
						@@java_pid, @@stdin, @@stdout, @@stderr  =  external_open("java -jar DataPreparation.jar")
						debug "java pid = " + @@java_pid.to_s, 3
					end
					@@java_thread.join
					java_ret = @@stdout.read + " " + @@stderr.read
					##############################################
					##############################################
					##############################################
					
					
					# java has finished. Get the new size of the log-file: This is the part of the logfile that corresponds to this action
					# and that will be inserted during output
					end_off = File.size?(@@import_ctslogfile).to_i
					File.open(@@import_rubylogfile, "a") do |file|
						java_feedback = File.read(@@import_ctslogfile, end_off - start_off, start_off) if File.exist? @@import_ctslogfile
						file.write "[INSERT] #{timestamp} file=#{@@import_ctslogfile} endoffset=#{end_off.to_s}\n" 
						java_msg = "#{timestamp} java return = #{java_ret.gsub("\n", "<br>")}\n"
						# write some feedback about the overall process
						if @@java_thread_got_killed
							file.write "  [WARN] #{timestamp} finished. Import has been canceled by the user!\n"
						else
							if @@error_phrases.any? { |phrase| java_feedback.include? phrase or java_ret.include? phrase}
								file.write " [ERROR] " + java_msg
								file.write " [ERROR] #{timestamp} finished. Something went wrong! Check the output above!\n"
							elsif @@warn_phrases.any? { |phrase| java_feedback.include? phrase or java_ret.include? phrase}
								file.write "  [WARN] " + java_msg
								file.write "  [WARN] #{timestamp} finished. Some warnings occured during the import! Check the output above!\n"
							else
								file.write "  [INFO] " + java_msg
								file.write "  [SUCC] #{timestamp} finished. Data imported successfully for '#{@@import_ctsname}'\n"
							end
						end
						# end marker of this action
						file.write "   [END] #{timestamp}\n" 
					end
					
					# finished import - reset variables
					@@import_ctsname 	= nil
					@@import_ctsdir 	= nil
					@@import_ctslogfile = nil
					debug "finished import!", 2
				rescue Exception => e
					error "while importing data to cts", e
					File.open(@@import_rubylogfile, "a") do |file|
						file.write " [ERROR] #{timestamp} finished. Error while executing 'java -jar DataPreparation.jar'. #{e.message}\n" 
						file.write "   [END] #{timestamp}\n"
					end
				ensure
					# remove lock even on errors
					File.delete( @@import_rubylock )
				end
			end
			
			# sleep one second for the command to be executed in the right directory until it will be set back
			sleep 1
		else
			File.open(@@import_rubylogfile, "a") do |file| 
				file.write " [ERROR] #{timestamp} finished. Another import process is already running!\n"
				file.write "   [END] #{timestamp}\n"
			end
			ret = {success: false, msg: "[ERROR] Another import process is already running!"}
		end
	rescue  => e
		error "while importing data to cts (outer)", e
		File.open(@@import_rubylogfile, "a") do |file| 
			file.write " [ERROR] #{timestamp} finished. #{e.message}\n"
			file.write "   [END] #{timestamp}\n"
		end
		ret = {success: false, msg: "[ERROR] " + e.message}
	ensure
		# restore base-dir. ensure this even if an error occured!
		Dir.chdir(@@import_rubybasedir)
	end
	
	return ret.to_json
end


post '/ajax_cancel_import' do
	ret = {succ: false, msg: ""}
	
	# thread could only be canceled if its active
	begin
		if @@java_thread and @@java_pid
			debug "now canceling java-thread", 2
			# kill the thread and set a killed flag
			@@java_thread_got_killed = true
			# this is the important part !!! 9 = "EXIT"
			Process.kill(9, @@java_pid) 
			# kill also the thread. but this alone would not kill the spawned external program
			@@java_thread.kill 
			
			# feedback
			ret[:succ] = true
			ret[:msg] = 'Import has been canceled. Please <a href="' + url("/forcts/#{params[:name]}/action/import") + '" class="alert-link">reload</a> the page'
		else
			debug "no java-thread or PID exists", 3
			ret[:msg] = "No Import is running"
		end
	rescue Exception => e
		error "while canceling import", e
		ret[:msg] = e.message
		# remove lock-file
		File.delete( @@import_rubylock )
	end
	
	return ret.to_json
end




post '/ajax_create_browser' do
	ret = {succ: false, msg: "Could not copy the template 'browser' to the tools folder!"}
	begin
		# check for top-level tools folder
		if not File.exist?( get_tools_path() )
			Dir.mkdir( get_tools_path() )
		end
		# check fpr browser folder
		if not File.exist?(File.join(get_tools_path(), "browser"))
			FileUtils.cp_r( File.expand_path(File.join("resources", "browser")), 
							File.expand_path(File.join(get_tools_path(), "browser")) )
			sleep 9
			ret = {succ: true, msg: 'Browser has been created! Please <a href="' + url("/forcts/#{params[:name]}/action/browse_browser") + '" class="alert-link">reload</a> this page.'}
		else
			debug "Browser already exists. No need to create it!", 3	
		end
	rescue Exception => e
		ret[:msg] = "[ERROR] " + e.message
	end
	
	return ret.to_json
end






post '/ajax_create_alignbrowser' do
	ret = {succ: false, msg: "Could not copy the template 'alignbrowser' and/or 'Alignment' to the tools folder!"}
	begin
		# check for top-level tools folder
		if not File.exist?( get_tools_path() )
			Dir.mkdir( get_tools_path() )
		end
		# check for the align browser
		if not File.exist?(File.join(get_tools_path(), "alignbrowser"))
			FileUtils.cp_r( File.expand_path(File.join("resources", "alignbrowser")), 
							File.expand_path(File.join(get_tools_path(), "alignbrowser")) )
			sleep 9
		else
			debug "'alignbrowser' already exists. No need to create it!", 3	
		end
		# check for the traviz tool
		if not File.exist?(File.join(get_tools_path(), "Alignment"))
			FileUtils.cp_r( File.expand_path(File.join("resources", "Alignment")), 
							File.expand_path(File.join(get_tools_path(), "Alignment")) )
			sleep 9
		else
			debug "'Alignment' already exists. No need to create it!", 3	
		end
		
		ret = {succ: true, msg: 'Alignment-Browser has been created! Please <a href="' + url("/forcts/#{params[:name]}/action/browse_align") + '" class="alert-link">reload</a> this page.'}
	rescue Exception => e
		ret[:msg] = "[ERROR] " + e.message
	end
	
	return ret.to_json
end




post '/ajax_create_parallelbrowser' do
	ret = {succ: false, msg: "Could not copy the template 'parallelbrowser' and/or 'Alignment' to the tools folder!"}
	begin
		# check for top-level tools folder
		if not File.exist?( get_tools_path() )
			Dir.mkdir( get_tools_path() )
		end
		# check for the parallel browser
		if not File.exist?(File.join(get_tools_path(), "parallelbrowser"))
			FileUtils.cp_r( File.expand_path(File.join("resources", "parallelbrowser")), 
							File.expand_path(File.join(get_tools_path(), "parallelbrowser")) )
			sleep 9
			ret = {succ: true, msg: 'Parallel-Browser has been created! Please <a href="' + url("/forcts/#{params[:name]}/action/browse_parallel") + '" class="alert-link">reload</a> this page.'}
		else
			debug "Parallel-Browser already exists. No need to create it!", 3	
		end
	rescue Exception => e
		ret[:msg] = "[ERROR] " + e.message
	end
	
	return ret.to_json
end



post "/ajax_reset_activities" do
	begin
		if not File.exist?( @@import_rubylock )
			# import is not running currently. Check if file exists
			if File.exists? @@import_rubylogfile
				File.delete @@import_rubylogfile
			end
		else
			# import is running. keep the messages from the current import
			if File.exists? @@import_rubylogfile
				s = File.read @@import_rubylogfile
				new_s = " [START]" + s.split(" [START]").last
				File.open(@@import_rubylogfile, "w") do |f|
					f.write new_s
				end
			end
		end
	rescue Exception => e
		error "while resetting recent activities", e
	end
	
	return ""
end


post "/ajax_delete_tools" do
	ap params
	ret = {succ: false, msg: ""}
	begin
		case params["tool"]
		when "del_tool_browser"
			FileUtils.rm_rf( File.join(get_tools_path, "browser") )
			ret = {succ: true, msg: "The 'browser' has been removed successfully. It can be restored in any browse --> browser tab."}
		when "del_tool_alignment"
			FileUtils.rm_rf( File.join(get_tools_path, "Alignment") )
			ret = {succ: true, msg: "The 'Alignment' (TRAViz) has been removed successfully. It can be restored in any browse --> Alignmentbrowser tab."}
		when "del_tool_alignmentbrowser"
			FileUtils.rm_rf( File.join(get_tools_path, "alignbrowser") )
			ret = {succ: true, msg: "The 'alignbrowser' has been removed successfully. It can be restored in any browse --> Alignmentbrowser tab."}
		when "del_tool_parallelbrowser"
			FileUtils.rm_rf( File.join(get_tools_path, "parallelbrowser") )
			ret = {succ: true, msg: "The 'parallelbrowser' has been removed successfully. It can be restored in any browse --> Parallelbrowser tab."}
		when "del_tool_all"
			FileUtils.rm_rf( File.join(get_tools_path, "browser") )
			FileUtils.rm_rf( File.join(get_tools_path, "Alignment") )
			FileUtils.rm_rf( File.join(get_tools_path, "alignbrowser") )
			FileUtils.rm_rf( File.join(get_tools_path, "parallelbrowser") )
			ret = {succ: true, msg: "All tools have been removed successfully. They can be restored in any browse tabs."}
		end
	rescue Exception => e
		ret = {succ: false, msg: e.message}
	end

	return ret.to_json 
end



























# show the login screen
get '/auth/login' do
	erb :login
end

# when sending data back from the login screen
post '/auth/login' do
	# perform authentication
	env['warden'].authenticate!
	
	# store a flash message
	flash[:success] = env['warden'].message
	
	# try to go back to the back from where called. But not to the login page again
	if session[:return_to].nil?
		redirect to('/')
	elsif session[:return_to].include?('/auth/login')
		redirect to('/')
	else
		redirect session[:return_to]
	end
end


# logout from the current session
get '/auth/logout' do
	# logout
	env['warden'].raw_session.inspect
	env['warden'].logout
	# store a flash message
	flash[:success] = 'logged out successfully'
	# go to the index page. will show the login-screen again
	redirect to('/')
end


# when trying to enter a page where authentication is required but NOT loged in
post '/auth/unauthenticated' do
	# store the page which the user wanted to show
	session[:return_to] = env['warden.options'][:attempted_path]
	# store a flash message
	flash[:error] = env['warden'].message || "please log in"
	# redirect to the login page
	redirect to( '/auth/login' )
end


run! if app_file == $0

end # end class





################################################################################
#####  Archive of old code  ####################################################
################################################################################



=begin
	
	# get data to show
	if cts_url != ""
		begin
			site = Nokogiri::HTML( open( cts_url ).read ) 
			@xml = CGI.escapeHTML(site.to_xml)
		rescue
			@xml = "CTS is not responding. #{cts_url}"
		end
	else
		@xml = "No CTS instances are running on any Tomcat servers. I tried: \n  " + cts_tomcat_urls.join(" \n  ")
	end
	
	
<% if @collection or @action  %>
	<h1>ctsname = <%= @ctsname %></h1>
	<h1>action = <%= @action %></h1>
	
	<h1>GetCapabilities = </h1>
	<pre><%= @xml %></pre>
<% else %>
	<h1>Index Page</h1>
<% end %>
	

################################################################################
#####  trying various ports for tomcat cts #####################################
################################################################################

# # cts_url will store the correct url 
# cts_url = ""
# # all these urls will be tested for response
# cts_tomcat_urls = []
# cts_tomcat_urls.push "http://localhost:8080/cts/cts/?request=GetCapabilities"
# cts_tomcat_urls.push "http://localhost:9080/cts/cts/?request=GetCapabilities"
# cts_tomcat_urls.push "http://localhost:9090/cts/cts/?request=GetCapabilities"
# cts_tomcat_urls.push "http://localhost:9081/cts/cts/?request=GetCapabilities"

# cts_tomcat_urls.each do |url|
# 	begin
# 		open( url ).read	# try to open the url
# 		cts_url = url 		# if it does not fail, this line will be executed. store URL!
# 		debug "found the cts_url: #{url}" , 2		# feedback in green
# 	rescue => e 
# 		debug "this is not a cts_url: #{url}" , 3	# feedback in yellow
# 	end
# end


# ### needed for WAR-Files in Tomcat since root changes !!!
# @context_path = ""
# @context_path = $servlet_context.getContextPath if $servlet_context
# debug "@context_path = '#{@context_path}'", 2

	
=end
