require 'rubygems'			# only required for Ruby < 1.8
require 'bundler/setup'		# add gems from Gemfile to load-path
Bundler.require(:default, :development)	# actually loads all specified gems. Remove :development in production mode

begin
	# DataMapper.setup(:default, "sqlite3://#{Dir.pwd}/db.sqlite")
	# set up the connection to the database. Here: flat-file yaml database
	# db is the directory where the files are stored
	DataMapper.setup(:default, "yaml:#{Dir.pwd}/db")
	DataMapper::Model.raise_on_save_failure = true
	
	# define a user model
	class User
		include DataMapper::Resource
		include BCrypt
		
		# the id (autoincrement) 
		property :id, 			Serial, 		:key => true,		:unique => true
		# the username and password of the user 
		property :username, 	String, 		:length => 3..48,	:unique => true
		property :password, 	BCryptHash
		# and its role, e.g. "admin", "guest", "user"
		property :role, 		String, 		:length => 3..48
		
	 	# trying an authentification. test against saved password
		def authenticate(attempted_password)
			if self.password == attempted_password
				true
			else
				false
			end
		end
	end

	# write data to database
	DataMapper.finalize

	# there might be errors under JRuby with this when connecting to an completly new
	# (i.e. not existing) db.sqlite file. Work around: connect to the same database
	# with MRI ruby and add same data and delete the data again. Thus, the schema
	# has been written and this does not cause errors anymore. Or just copy the 
	# db.sqlite.bak to db.sqlite (= empty Database with schema: id, username, password)
	DataMapper.auto_upgrade! 

rescue Exception => e
	puts "[ERROR] #{e}\n#{e.backtrace.join("\n")}"
end

# create a new user using irb. Type these commands on a shell inside the directory
# where the model.rb file is located (=dir of this file) :
# rvm use ruby
# irb
# require "./model.rb"
# User.create(username: "cts", password: "cts", role:"guest")

# list all users:
# User.all 

# delete all users:
# User.destroy!