// shows an error message if something went wrong
function error(msg){
	console.log("[PARALLELBROWSER-ERROR] " + msg);
	$("#parallel_error_msg").text(msg);
	$("#parallel_error_div").css("display", "block");
};
// hides the error message. e.g. when a new action is triggered from the beginning
function reset_error(){
	$("#parallel_error_div").css("display", "none");
};




$.fn.moveUp = function() {
    $.each(this, function() {
         $(this).after($(this).prev());   
    });
};
$.fn.moveDown = function() {
    $.each(this, function() {
         $(this).before($(this).next());   
    });
};




function toggle_fold(level){
	// get the element with this data-level
	var el = $('*[data-level="' + level + '"]');
	console.log(el);
	
	// form the items with this class, which display-property should be toggled
	var cls = "fold__" + level.replace(/\./g, "__");
	$("." + cls).each(function(){
		// toggle if should be shown or not
		$(this).toggleClass("hidden");
	});
	
	// change the triangle facing direction
	el.find("span").toggleClass("glyphicon-triangle-bottom");
	el.find("span").toggleClass("glyphicon-triangle-right");
};




function build_urn(){
	// reset outputs
	$("#passage_link").html("&nbsp;");
	
	
	// get all the urns from the right candis
	var urns = [];
	$(".right_candi").each(function() {
		urns.push( $(this).attr("data-urn") );
	})
	console.dir(urns);
	
	// if there were any
	if(urns.length > 0){
		// http://ctstest.informatik.uni-leipzig.de/demo/plain/alignment?urn=urn:cts:songs:christmas.ohtennenbaum.cat.1:&alignurns=urn:cts:songs:christmas.ohtennenbaum.de.1:_urn:cts:songs:christmas.ohtennenbaum.fr.1
		// build url: take first urn as "urn" and rest as "alignurns"
		var parallel_url = cts_url + "/plain/alignment?urn=" + urns[0];
		var first_u = urns[0];
		load_levels(urns[0]);
		
		// get the sub-level
		var sub_level = $(".level_item.active").attr("data-level");
		console.log("sub_level = " + sub_level)
		if(sub_level !== "ALL" && typeof sub_level != 'undefined' ){
			parallel_url += sub_level;
			first_u += sub_level;
		}
		
		// generate the align url
		urns.splice(0, 1);
		parallel_url += "&alignurns=" + urns.join( cts_sep );
		// generate passage link
		$("#passage_link").prop("href", parallel_url);
		$("#passage_link").text( $("#passage_link").prop("href") );
		
		// generate the fullscreen url
		fl = first_u;
		if(urns.length > 0){
			fl += "___" + urns.join("___");
		}
		$("#a_fullscreen").attr("href", "");
		var bas = $("#a_fullscreen").prop( "href" );
		bas = bas.split("&urns=")[0];
		$("#a_fullscreen").attr("href", bas + "&urns=" + fl );
		
		// generate download link
		var download_name = "parallel_alignment.csv";
		try{
			var download_name = (first_u + "___" + urns.join("__")).replace(/:/g, "_") + ".csv";
		} catch (e) { }
		$("#a_download_csv").prop("href", parallel_url);
		$("#a_download_csv").prop("download", download_name);
	} else {
		load_levels("");
	} // if urns.length > 0
	
}; // end build urn



function load_levels(align_candi_urn){
	if(align_candi_urn === ""){
		// remove all items
		$('#parallel_level_list').html("");
	}
	
	// check if something to do and if top most URN has changed
	if(align_candi_urn !== "" && old_candi_urn !== align_candi_urn){
		// build the url to get all sentence candidates
		// http://ctstest.informatik.uni-leipzig.de/dev/plain/urnstypestext?urn=urn:cts:pbc:hif.hif: 
		var sentences_candidates_url = cts_url + "/plain/urnstypestext?urn=" + align_candi_urn;
		console.log("sentences_candidates_url=" + sentences_candidates_url);
		// remove all items
		$('#parallel_level_list').html("");
		// get a list of all sentences for this alignment candidate via GET request
		jQuery.ajax({
				url: sentences_candidates_url,
			success: function( sentence_candis ) {
				
				// check if there were any sentence candidates
				if(sentence_candis !== ""){
					// add an "all" option
					var sentence_list = sentence_candis.split("\n");
					first_cls = "";
					if(cts_sublevel === "!_#^^<<>>"){
						first_cls = " active ";
					}
					$("#parallel_level_list").append('<a id="level_0" class="list-group-item list-group-item-default level_item clickable ' + first_cls + '" data-level="ALL">all (' + sentence_list.length + ' lines)</a>');
					
					// init some temporary variables
					var line_arr = [];	// temp. array of the splitted entries for one line
					var sub_urn  = "";	// the reduced sub-urn of this sentence (without the alignment-candidates urn!)
					var sub_type = "";	// type could be chapter, book, sentence, ...
					var sub_len  = "";	// length of the entry. Only make entries clickable that have a length (and not "-")
					var cls 	 = ""; 	// mark entries as clickable by adding a custom clickable class
					var indent   = "";	// indent according to urn-depth
					
					// list all found sentence candidates
					for (var i = 0; i < sentence_list.length; i++) {
						// split the line at tabs. It will then have 3 entries
						line_arr = sentence_list[i].split("\t");
						// 1. the full urn of this sentence. remove the candidates urn to only store the sentence specific identifier
						sub_urn  = line_arr[0].replace(align_candi_urn, "");
						// read the type of this line. could be chapter, sentence, ...
						sub_type = line_arr[1];
						// and read the length-information
						sub_len  = line_arr[2];
						
						// if this line has a length, it could be displayed in the alignment-TRaviz and thus a clickable entry 
						if(sub_len !== "-"){
							cls = " clickable";
						} else {
							// if not, it could be used for a clickable fold hierarchy
							cls = " foldable";
						}
						
						// hide everything on initial display but the topmost level
						if(sub_urn.indexOf(".") >= 0){
							// hide but setting the hidden class
							cls += " hidden";
							// add a class which describes the upper fold-button-level which
							// a click on a button with this level will toggle the hidden status. this makes fold and unfold very easy and efficient!
							// just remove the last entry after the dot to get the upper level
							cls += " fold__" + sub_urn.slice(0, sub_urn.lastIndexOf(".") ).replace(/\./g, "__");
						}
						
						if(sub_urn === cts_sublevel){
							cls += " active ";
						}
						
						// convert the depth=tabs into nice indentation
						indent   = "";
						for (var k = 0; k < sub_urn.split(".").length - 1; k++) {
							indent += "&nbsp;&nbsp;&nbsp;&nbsp;";
						};
						
						// do not print "(-)" for book or chapter
						var print_len = ' (' + sub_len + ')';
						if( sub_len === "-"){
							//  !!! IMPORTANT the glyphicon-triangle-bottom is used to indicate if folded or not !!!
							print_len = ' <span class="glyphicon glyphicon-triangle-bottom"></span>';
						}
						
						// add a list entry
						if(sub_urn != ""){
							$("#parallel_level_list").append('<a class="list-group-item list-group-item-default level_item' + cls + '" data-urn="' + line_arr[0] + '" data-level="' + sub_urn + '">' + indent + '('+sub_urn+') ' + sub_type + print_len + '</a>');
						}
					}
					
					if(cts_sublevel  !== "!_#^^<<>>"){
						// unfold stuff
						var num_arr = cts_sublevel.split(".");
						var lvl = "";
						for (var k = 0; k < num_arr.length-1; k++) {
							if(k > 0){
								lvl += ".";
							}
							lvl += num_arr[k];
							var el = $('*[data-level="' + lvl + '"]');
							if( el.find("span").hasClass("glyphicon-triangle-bottom") ){
								toggle_fold(lvl);
							}
						};
						// and after opening folds, scroll inside the list
						var container = $('#parallel_level_list');
						var scrollTo  = $('*[data-level="' + cts_sublevel + '"]');
						container.scrollTop(
							scrollTo.offset().top - container.offset().top + container.scrollTop() - 40
						);
						
						// reset variables to only show on first call and not on regular clicks
						cts_sublevel  = "!_#^^<<>>";
					}
				} // end sentence_candis !== ""
			}, // success function
			error: function(){
				error("Could not get the sentence candidates, url = " + sentences_candidates_url);
			},
			async:   false
		});// sync ajax call

	} // end align_candi_urn !== ""
	
	// remember the last urn
	old_candi_urn = align_candi_urn;
}




// ################################################################################
// #####  Func Ready  #############################################################
// ################################################################################





function func_ready() {
	
	// ################################################################################
	// #####  Done at Startup  ########################################################
	// ################################################################################
	
	// the old top most right item. init with random urn that will never been seen in the wild
	old_candi_urn = "!_#^^<<>>";
	cts_sublevel  = "!_#^^<<>>";
	
	// get the name of the CTS. The name is given via the URL
	params_arr = window.location.search.split('&');
	console.dir(params_arr);
	cts_url = params_arr[0].split('=')[1];
	console.log("cts_url = " + cts_url);
	cts_sep = params_arr[1].split('=')[1];
	console.log("cts_sep = " + cts_sep);
	if(params_arr.length > 2){
		cts_urns = params_arr[2].split('=')[1];
	} else {
		cts_urns = ""
	}
	console.log("cts_urns = " + cts_urns);
	
	// build the url to ask for the alignment candidates for this CTS
	// http://ctstest.informatik.uni-leipzig.de/pbc/plain/alignmentcandidates
	var parallel_candi_url = cts_url + "/plain/alignmentcandidates";
	console.log("parallel_candi_url = " + parallel_candi_url);
	
	// get the alignment candidates for this specific CTS via GET command
	$.get( parallel_candi_url )	
		// if GET was successfull
		.done( function( parallel_candis ) {
			// reset error if there were errors before
			reset_error();
			// empty the left list of alignment candidates
			$("#parallel_candi_list").html("");
			// check, if this CTS has any candidates
			if(parallel_candis !== ""){
				// fill the left list with all the candidates
				parallel_candis.split("\n").forEach(function(str){
					// filter for empty lines, e.g. at the end of the transmission
					if(str !== ""){
						// some candidates are indented. BUT this MUST NOT be used within urn-requests
						var urn = str.replace(/\t/g, "");
						// some candidates are indented. For better HTML rendering, convert to non-breakables spaces
						var txt = str.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
						// add a new list item and add the urn as a data-field
						$("#parallel_candi_list").append('<a class="list-group-item list-group-item-default parallel_candi" data-urn="' + urn + '">' + txt + '</a>')
					}
				});
			} else {
				// if there were no alignment candidates, show a short message in the list
				$("#parallel_candi_list").append('<a class="list-group-item list-group-item-default">No parallel (alignment) candidates found</a>');
			}
		})  
		// in case of errors during the conncetion show the error
		.fail(function() {
			$("#parallel_candi_list").html('<a class="list-group-item list-group-item-default">Could not connect to: ' + parallel_candi_url + '</a>');
			error("could not connect to: " + parallel_candi_url);
		});
	
	
	// ################################################################################
	// #####  Click-Event for left list  ##############################################
	// ################################################################################
	
	if(cts_urns !== ""){
		c_arr = cts_urns.split("___");
		
		// add in the middle col
		for (var i = 0; i < c_arr.length; i++) {
			c_urn = c_arr[i].slice(0, c_arr[i].lastIndexOf(":")) + ":";
			var el = $("<a>").prop("id", "notneeded").addClass("list-group-item list-group-item-default right_candi").attr("data-urn", c_urn).text( c_urn );
			$("#parallel_urn_list").append(el);
		};
		
		// load right col
		cts_sublevel = c_arr[0].slice( c_arr[0].lastIndexOf(":")+1, c_arr[0].length );
		console.log("cts_sublevel = " + cts_sublevel);
		
		// build urn
		build_urn()
	}
	
	
	// handle clicks on the alignment-candidates 
	// 		-> Show sentence candidates in the right list
	// use delegation since alignment-candidates are added dynamically at startup
	$("#parallel_candi_list").on('click', '.parallel_candi', function() {
		// reset error if there were errors before
		reset_error();
		// set active flag for only the clicked alignment-candidate entry
		$(".parallel_candi").removeClass("active");
		$(this).addClass("active");
	});
	
	$("#parallel_candi_list").on('dblclick', '.parallel_candi', function() {
		// click is also called, thus also code is executed
		var el = $("<a>").prop("id", "notneeded").addClass("list-group-item list-group-item-default right_candi").attr("data-urn", $(this).attr("data-urn")).text( $(this).text() );
		$("#parallel_urn_list").append(el);
		build_urn()
	});
	$("#btn_add").on('click', function() {
		// click is also called, thus also code is executed
		var act = $(".parallel_candi.active");
		var el = $("<a>").prop("id", "notneeded").addClass("list-group-item list-group-item-default right_candi").attr("data-urn", act.attr("data-urn")).text( act.text() );
		$("#parallel_urn_list").append(el);
		build_urn()
	});
	
	
	
	
	// ################################################################################
	// #####  Click-Event for right list  #############################################
	// ################################################################################
	
	
	// handle clicks on the sentence-candidates
	// 		-> Show in graphical alignment frame
	// use delegation since sentence candidates are added dynamically
	$("#parallel_urn_list").on('click', '.right_candi', function() {
		// reset error if there were some before
		reset_error();
		// set active flag for the clicked entry
		$(".right_candi").removeClass("active");
		$(this).addClass("active");
	});
	
	$("#parallel_urn_list").on('dblclick', '.right_candi', function() {
		// click is also called, thus also code is executed
		$(this).remove();
		build_urn()
	});
	$("#btn_remove").click(function(){
		$(".right_candi.active").remove();
		build_urn()
	});
	
	
	
	
	// ################################################################################
	// #####  up and down  ############################################################
	// ################################################################################
	
	
	$("#btn_up").click(function(){
		$(".right_candi.active").moveUp();
		build_urn()
	});
	$("#btn_down").click(function(){
		$(".right_candi.active").moveDown();
		build_urn()
	});
	
	// ################################################################################
	// #####  Action  #################################################################
	// ################################################################################
	
	
	
	$("#btn_start").click( function() {
		build_urn();
		// reset table
		$("#div_result").html("");
		// show table
		// perform GET to load data
		$.get( $("#passage_link").prop("href") )	
			// if GET was successfull
			.done( function( res ) {
				var show_warning = $("#chk_warnings").is(':checked');
				var limit = 50;
				var conf = true;
				if( show_warning && res.length > limit*1024) {
					conf = confirm("The result is larger than " + limit + "kb (" + (res.length/1024).toFixed(2) + " kb). The table rendering could take some while. Do you like to show the table anyway?");
				}
				if( conf ){
					// construct a new table object
					var table = $('<table>').addClass('table table-striped table-condensed ');
					table.append( $("<colgroup>").append( $("<col>").prop("width", "50px")) );
					
					// add the table headings
					var th_row = $("<tr>");
					var obj = res.split("\n")[0].split("\t");
					for (var i = 0; i < obj.length; i++) {
						th_row.append( $("<th>").text(obj[i]) );
					}
					table.append( $("<thead>").append(th_row) );
					
					// add each row
					var tbody = $("<tbody>");
					var rows = res.split("\n");
					for (var i = 1; i < rows.length; i++) {
						var trow = $("<tr>")
						var obj = rows[i].split("\t");
						for (var k = 0; k < obj.length; k++) {
							trow.append( $("<td>").text(obj[k]) );
						};
						tbody.append(trow);
					};
					table.append( tbody );
					
					// show table in result div
					$("#div_result").html(table);
				} // show table inner (length)
			})
			// in case of errors during the conncetion show the error
			.fail(function() {
				error("could not connect to: " + parallel_url);
			});
	});


	// handle clicks on the foldable-items
	// 		-> fold / unfold stuff
	// use delegation since items are added dynamically
	$("#parallel_level_list").on('click', '.foldable', function() {
		// get the level of the current item
		var level = $(this).attr("data-level");
		// toggle the fold status
		toggle_fold(level);
		// set active
		$(".level_item").removeClass("active");
		$(this).addClass("active");
		// build new urn
		build_urn();
	});
	
	$("#parallel_level_list").on('click', '.clickable', function() {
		// set active
		$(".level_item").removeClass("active");
		$(this).addClass("active");
		// build new urn
		build_urn();
	});




	// regexp filter while typing in the search field
	// use the keyup event because then we get the final input (in contrast to keypress or keydown)
	$("#txt_regex").on("keyup", function(){
		// read the regexp
		var reg_string = $("#txt_regex").prop("value");
		// i - case-insensitive ; g - global ; m - multi
		var regexp = new RegExp(reg_string, "i");
		// show matching candidates. no need to check for empty regexp, will match all anyway
		// for each candidate in the list
		$(".parallel_candi").each(function(){
			// test if matching
			if( $(this).text().search(regexp) >= 0 ){
				$(this).css("display", "block");
			} else {
				$(this).css("display", "none");
			}
		});
	});
	
}




// bind to the ready and pagechange function (for AJAX requests)
$(document).ready( func_ready )
$(document).on("pagechange", func_ready )


// bind keydown events for going left or right (left or right arrows)
$(document).keydown(function(e) {
    console.log(e.keyCode);
    // DEL
    if(e.keyCode === 46){
    	$("#btn_remove").trigger("click");
    }
});







					